# The data structure we will be using most often is a data.frame.
# That's because ggplot2 requires data.frame, and read.csv reads into a data.frame
# Think of it like a spreadsheet table, where each column is a vector

x= 10:1
y= -4:5
z= c("blue","red","yellow","green","orange","purple","blue","green","blue","red")

x; y; z  # you can also separate statements with a semicolon

# so far, these are just separate vectors

myDF = data.frame(x,y,z)
myDF

zFactor = as.factor(z)
zFactor  #zFactor is shorter than Z; no repeat values

myDF2 = data.frame(x,y,zFactor)  # nevertheless, this works
myDF2

# since data.frames are objects (as is everything in R, it has properties (functions, actually)

nrow(myDF)
ncol(myDF)
dim(myDF)
names(myDF) #but I didn't specify names!!  So they're just the names of the vectors!

# so let's assign better names:
colnames(myDF)= c("sequence","score","color")
colnames(myDF)
# you can assign rownames too--a name for each observation
myDF #should display the new column names

# You can also index into the data.frame:
myDF[3] # returns the third column
names(myDF)[3]

# a data.frame usually have many observations (rows).  Therefore you can display head/tail
head(myDF)
tail(myDF)
head(myDF, n=4)


class(myDF)
# each column of the data.frame is its own vector.  Can be accessed as a variable of the DF:
myDF$z   # doesn't work, because within the data.frame, we renamed this column to "color"
myDF$color #this works
z  # but the underlying vector that we had defined before adding it to the data.frame is still valid
# and as we said earlier, you can still use the brackets...
myDF[3]
# in fact, since it it 2-dim, you can use two indices:
myDF[4,3]  # 4th row, 3rd column. Interesting, shows z as a factor--it's really not.  converts!
myDF2[4,3]
class (myDF[3])
class(z)  # see, it's not a factor!
myDF[3]  !
myDF [4,3] # but look at the display.  freaky.  But if you show just that column, it returns
#           it has a factor vector.

# you can also specify a range of rows or columns:
myDF[2:4, 2:3] # rows 2-4 and columns 2-3
myDF
y

# when using ranges, you can use specify all:
myDF[, 2:3] # all rows, columns 3-4
# or you can use column names:
myDF[,c("score","color")]

#recall that myDF2 has a factor (zFactor) as its third vector
myDF2
class (myDF[3])         # returns data.frame
class (myDF["color"])   # returns data.frame
class (myDF[,3])        # returns factor (and that vector wasn't defined as a factor!)
class (myDF[,"color"])  # returns factor (and that vector wasn't defined as a factor!)
# R converts character vectors to factors inside of a data.frame
# now let's try it with myDF2 which has a factor for the third vector
colnames(myDF2)= c("sequence","score","color")
class (myDF2[3])         # returns data.frame
class (myDF2["color"])   # returns data.frame
class (myDF2[,3])        # returns factor 
class (myDF2[,"color"])  # returns factor 

# and there are also double [[]]
class(myDF["color"])  # returns a data frame
class(myDF[["color"]])# returns a factor

# so be careful!!  If you want to know about this, read this link:
# https://stackoverflow.com/questions/1169456/the-difference-between-and-notations-for-accessing-the-elements-of-a-lis


# There are also lists (elements can be different types)
# There are also matrices (all elements of the same type, support matrix arithmetic)
# You don't need variable names for matrix
# There are also arrays (multidimensional vectors)