# A vector is a collection of elements, all of the same data type
# R is a vectorized language, meaning, it will apply operations to 
# entire vectors without explicit looping!!

x <- c(1,2,3,4,5,6,7,8,9,10)  # c means combine
x

x+5
x-3
x^2
sqrt(x)

# comparing vectors
 y<- -5:4  # creates a vector with integers ranging from -5 through 4
 x+y
 x/y # note division by zero is Inf (a special constant)
 
 # properties of vectors
 length(x)
 
 x
  x + c(1,2) # adds 1 and 2 alternately to each element in the vector
  
  x + c(1,2,3) #note the warning--c(1,2,3)not a length multiple of x
  
  # logical comparisons of vectors
  x<-c(1,2,3,4,5,1,2,3,4,5)
  y<-c(1,2,3,4,5,6,7,8,9,10)
  x<y
  y<x
any(x<y)

# indvidual elements of a vector are indexted by using square brackets []

y[3]   # accesses the third element of vector y
y[4:7] #accesses the fourth through seventh elements of vector y
y[c(4,7)]  #access the fourth and seventh elements of vector y

# give your vector elements meaningful names
# like an associative array

x<-c(Candy = "Kit Kat", IceCream = "Rocky Road", Chips = "Fritos")
x

# Or, you can supply names after the fact
y<- c(73,95,82)
y
names(y)<- c("My Grade", "Your Grade", "Her Grade")
y

# factor vectors (categorical variables, or discretized)
# you can have a plain character vector:

favColors<- c("blue","red","green","yellow","green","blue","orange","blue")
favColors

favColorsFactors<-as.factor(favColors)
favColorsFactors #note non-repeats, and "levels", which are values

yearCollege<-c("freshman","sophomore","junior","senior")
yearCollege
yearCollegeFactors<-as.factor(yearCollege)
yearCollegeFactors

# note that the factors are listed alphabetically.  
# However, they have an order that is important (ordinal values)
# If defining the factor in the factor function, you must use <- and not =.
# I don't know why.
factor(yearCollegeFactors<-c("Freshman","Sophomore","Junior","Senior"),
       levels=c("Freshman","Sophomore","Junior","Senior"),
       ordered=TRUE)
yearCollegeFactors
 
factor(z<-c("hi","med","lo"))
z

# You can also use a vector that has been defined earlier:

factor(yearCollegeFactors2<-yearCollege,
       levels=c("Freshman","Sophomore","Junior","Senior"),
       ordered=TRUE)
yearCollegeFactors2

# Let's play with vectors: (http://r-statistics.co/R-Tutorial.html)
# creating vectors:
vec1 <- c(10, 20, 15, 40)  # numeric vector
vec2 <- c("a", "b", "c", NA)  # character vector
vec3 <- c(TRUE, FALSE, TRUE, TRUE)  # logical vector
vec4 <- gl(4, 1, 4, label = c("l1", "l2", "l3", "l4"))  # factor with 4 levels

# referencing vectos or elements of a vector
length(vec1)  # 4
print(vec1[1])  # 10
vec1[1]        # same as print(vec[1])
print(vec1[1:3])  # 10, 20, 15

# You do not have to declare or initialize a vector
# But if you expect a big one, it's more efficient:
numericVector <- numeric(100) # length 100 elements

# subsetting:
logic1 <- vec1 < 15  # create a logical vector, TRUE if value < 15
vec1[logic1]  # elements in TRUE positions will be included in subset
vec1[1:2]  # returns elements in 1 & 2 positions.
vec1[c(1,3)]  # returns elements in 1 & 3 positions
vec1[-1]  # returns all elements except in position 1.
          # Note that this is different from Python!!

# Sorting a vector
sort(vec1)  # ascending sort
sort(vec1, decreasing = TRUE)  # Descending sort 

# Sorting can also be achieved using the order() function 
# which returns the indices of elements in ascending order.

vec1[order(vec1)]  # ascending sort
vec1[rev(order(vec1))]  # descending sort



seq(1, 10, by = 2)  # diff between adj elements is 2
seq(1, 10, length=25)  # length of the vector is 25
rep(1, 5)  # repeat 1, five times.
rep(1:3, 5)  # repeat 1:3, 5 times
rep(1:3, each=5)  # repeat 1 to 3, each 5 times.

vec2 <- c("a", "b", "c", NA)  # character vector
is.na(vec2)  # missing TRUE
!is.na(vec2)  # missing FALSE
vec2[!is.na(vec2)]  # return non missing values from vec2

# calling functions:  use parentheses, use Help for arguments
# Help:  either use the Help window search box, or ?functionName in the console
# use aprorpos for a wildcard search
apropos("mea")
